// UrsAds1x15
// Erweiterte Alternative zu der Adafruit-Library: // https://www.adafruit.com/product/1083
//
// Autor: http://UllisRoboterSeite.de
// Doku:  http://bienonline.magix.net/public/esp8266-UrsWiFi.html
// Created: 2018-02-10
//
// Version 1.0 (2018-02-10)
// - Basis-Version
//
// Version 1.1 (2018-03-01)
// - readRegister mit ICACHE_RAM_ATTR versehen
// - umgestellt auf UrsTwi
// - Fehler bei Ads1x15Gain++ und Ads1x15Gain-- behoben
//
// Version 1.2 (2018-03-01)
// - getGainString() hinzugefgt
// 
//
// Version 1.3 (2020-04-12)
// - reset hinzugefgt
// - Ads1x14Address umbenannt in Ads1x15Address
// - ADS1015_REG_... umbenannt in ADS1X15_REG_...
// - attachInterrupt nach Schreiben des Konfigutionsregisters
//   verhindert versentliches auslsen der ISR
// - Fr Arduino erweitert
//
// Version 1.4 (2020-04-13)
// - Werte fr Ads1115Sps waren vollkommen falsch. Korrigiert.
//
// Version 1.5 (2020-04-14)
// - readRegister hat den bergebenen Registerparamter ignoriert
//   und immer nur das Conversion-Register ausgelesen.
//
// Version 1.6 (2020-04-14)
// - setMaxSps hinzugefgt.
//

#ifndef _URSADS1X15_h
#define _URSADS1X15_h

#include <Arduino.h>
#if defined(ESP8266)
   #include <UrsTwi.h>
#elif defined(__AVR__) 
   #include <Wire.h>
#define _twi Wire
#else
   #error "processor not supported"
#endif 


// =========================================================================
// IC-Adress-Varianten fr die ADS1x15-Module
// -------------------------------------------------------------------------
enum class Ads1x15Address : uint8_t {
  Default = 0x48, // 1001 000 (ADDR = GND)
  VDD = 0x41,     // 1001 001 (ADDR = VDD)
  SDA = 0x42,     // 1001 010 (ADDR = SDA)
  SCL = 0x43      // 1001 011 (ADDR = SCL)
};
// =========================================================================
// Konstanten fr die Multiplexer-Auswahl
// -------------------------------------------------------------------------
enum class Ads1x15Mux: uint16_t {
  Diff_0_1 = 0x0000,  // Differential P = AIN0, N = AIN1 (default)
  Diff_0_3 = 0x1000,  // Differential P = AIN0, N = AIN3
  Diff_1_3 = 0x2000,  // Differential P = AIN1, N = AIN3
  Diff_2_3 = 0x3000,  // Differential P = AIN2, N = AIN3
  Single_0 = 0x4000,  // Single-ended AIN0
  Single_1 = 0x5000,  // Single-ended AIN1
  Single_2 = 0x6000,  // Single-ended AIN2
  Single_3 = 0x7000   // Single-ended AIN3
};

// =========================================================================
// Konstanten fr die Verstrkung
// -------------------------------------------------------------------------

// Die Eingangsspannung darf (VDD + 0.3V) nicht berschreiten!
enum class Ads1x15Gain : uint16_t {
  Gain_2_3 = 0x0000,  // +/-6.144V range = Gain 2/3  0.1875mV (default)
  Gain_1   = 0x0200,  // +/-4.096V range = Gain 1    0.125mV
  Gain_2   = 0x0400,  // +/-2.048V range = Gain 2    0.0625mV
  Gain_4   = 0x0600,  // +/-1.024V range = Gain 4    0.03125mV
  Gain_8   = 0x0800,  // +/-0.512V range = Gain 8    0.015625mV
  Gain_16  = 0x0A00,  // +/-0.256V range = Gain 16   0.0078125mV
};

extern Ads1x15Gain& operator++(Ads1x15Gain&); // ++A
extern Ads1x15Gain  operator++(Ads1x15Gain &, int); // A++
extern Ads1x15Gain& operator--(Ads1x15Gain&); // --A
extern Ads1x15Gain  operator--(Ads1x15Gain&, int);  // A--

class UrsAds1x15 {
protected:
#ifdef ESP8266
   UrsTwi&     _twi;              // TwoWire-Interface
#endif // ESP8266
   uint8_t     _i2cAddress;       // Eingestellte IC-Adresse
   uint8_t     _conversionDelay;  // Zeitspanne in ms auf die fr eine Konvertierung gewartet wird
   Ads1x15Gain _gain;             // Aktuell eingestellter Verstrkungsfaktor
   uint16_t    _SamplesPerSecond; // Messungen pro Sekunde
   float       _mVoltsPerBit;     // Auf _gain beruhender Umrechnungsfaktor

   bool        _isContinuousMode = false;// true, wenn der kontinuierliche Modus aktiv ist
   uint16_t    _SpsCode;          // SPS-Code als Zahl.
   uint8_t     _alertPin;         // Pin fr Alert/RDY

   void        writeRegister(uint8_t reg, uint16_t value);  // Schreibt in das angegebene Register des ADS1x15 
   uint16_t    readRegister(uint8_t reg);                   // Liest das angegebene Register aus

#if defined(ESP8266)
   UrsAds1x15(UrsTwi& twi, Ads1x15Address i2cAddress);   // Initialisiert eine neue Instanz der UrsAds1x15Class-Klasse
#elif defined(__AVR__) 
   UrsAds1x15(Ads1x15Address i2cAddress);   // Initialisiert eine neue Instanz der UrsAds1x15Class-Klasse
#endif 


 public:
   void        reset();                         // Versetzt den Chip in den Power-Down-Zustand
   void        setGain(const Ads1x15Gain gain); // Legt den Verstrkungsfaktor fest
   Ads1x15Gain getGain(void) const;             // Liefert den aktuell eingestellten Verstrkungsfaktor
   String      getGainString();
   float       getMilliVoltsPerBit(void) const { return _mVoltsPerBit; } // Liefert den aktuellen Umrechnungsfaktor

   uint8_t     getConversionDelay(void) const { return _conversionDelay; }     // Liefert die Zeitspanne in ms, auf die fr eine Konvertierung gewartet wird
   uint16_t    getSamplesPerSeconds(void) const { return _SamplesPerSecond; }  // Liefert die Messrate (Messungen pro Sekunde)
   virtual void setMaxSps() = 0; // stellt die maximale Abtastrate ein.

   int16_t     readAdcRaw(const Ads1x15Mux mux);  // Fhrt eine Einzelmessung fr den angegebenen Kanal durch, liefert den Register-Rohwert
   float       readADCmV(const Ads1x15Mux mux) { return readAdcRaw(mux) * getMilliVoltsPerBit();}  // Fhrt eine Einzelmessung fr den angegebenen Kanal durch, liefert mV

   // Methoden zum Starten, Stoppen und Auslesen der kontunierlichen Wandlung
   void        startContinuousMode(const Ads1x15Mux mux, void(*ISR)(void), const uint8_t alertPin, const bool usePullUpForAlertPin = true);
   void        stopContinuousMode(void);
   int16_t     readContinuousModeRaw(void) const; // Liefert den Register-Rohwert
   bool        isContinuousMode(void) const { return _isContinuousMode; }
};

// ======================================================================================================
// ===  UrsAds1015
// ======================================================================================================
enum class Ads1015Sps : uint16_t {
  Sps128  = 0x0000,  //  128 samples per second
  Sps250  = 0x0020,  //  250 samples per second
  Sps490  = 0x0040,  //  490 samples per second
  Sps920  = 0x0060,  //  920 samples per second
  Sps1600 = 0x0080,  // 1600 samples per second (default)
  Sps2400 = 0x00A0,  // 2400 samples per second
  Sps3300 = 0x00C0   // 3300 samples per second
};

class UrsAds1015 : public UrsAds1x15 {
protected:
  Ads1015Sps  _Sps;  // Messungen pro Sekunde (Code)

public:
#if defined(ESP8266)
   UrsAds1015(UrsTwi& twi, Ads1x15Address i2cAddress = Ads1x15Address::Default);   // Initialisiert eine neue Instanz der UrsAds1015Class-Klasse
#elif defined(__AVR__) 
   UrsAds1015(Ads1x15Address i2cAddress = Ads1x15Address::Default);   // Initialisiert eine neue Instanz der UrsAds1015Class-Klasse
#endif 
  void        setSps(Ads1015Sps Sps);        // Legt die Anzahl der Messungen pro Sekunde fest
  Ads1015Sps  getSps(void) { return _Sps; }  // Liefert den Code fr die Messrate
  void        setMaxSps() { setSps(Ads1015Sps::Sps3300); } // stellt die maximale Abtastrate ein.
};


// ======================================================================================================
// ===  UrsAds1115
// ======================================================================================================
enum class Ads1115Sps : uint16_t {
  Sps8   = 0x0000, //   8 samples per second
  Sps16  = 0x0020, //  16 samples per second
  Sps32  = 0x0040, //  32 samples per second
  Sps64  = 0x0060, //  64 samples per second
  Sps128 = 0x0080, // 128 samples per second (default)
  Sps250 = 0x00A0, // 250 samples per second
  Sps475 = 0x00C0, // 475 samples per second
  Sps860 = 0x00E0  // 860 samples per second
};

class UrsAds1115 : public UrsAds1x15 {
protected:
  Ads1115Sps  _Sps; // Messungen pro Sekunde (Code)

public:
#if defined(ESP8266)
   UrsAds1115(UrsTwi& twi, Ads1x15Address i2cAddress = Ads1x15Address::Default);   // Initialisiert eine neue Instanz der UrsAds1115Class-Klasse
#elif defined(__AVR__) 
   UrsAds1115(Ads1x15Address i2cAddress = Ads1x15Address::Default);   // Initialisiert eine neue Instanz der UrsAds1115Class-Klasse
#endif 
  void        setSps(Ads1115Sps Sps);        // Legt die Anzahl der Messungen pro Sekunde fest
  Ads1115Sps  getSps(void) { return _Sps; }  // Liefert den Code fr die Messrate
  void        setMaxSps() { setSps(Ads1115Sps::Sps860); } // stellt die maximale Abtastrate ein.
};
#endif

